#include <sta/devices/raspi/gpio_pin.hpp>
#ifdef STA_RASPI_GPIO_ENABLED

#include <sta/debug/assert.hpp>
#include <sta/lang.hpp>

namespace sta 
{
    RaspiGpioPin::RaspiGpioPin(uint8_t pin, GpioMode mode) : pin_{ pin }, mode_{ mode } 
    {
        pinMode(pin, mode == GpioMode::GPIO_INPUT ? INPUT : OUTPUT);
    }

    void RaspiGpioPin::setState(GpioPinState state)
    {
        digitalWrite(pin_, state == GpioPinState::GPIO_LOW ? LOW : HIGH);
    }

    GpioPinState RaspiGpioPin::getState()
    {
        return digitalRead(pin_) == LOW ? GpioPinState::GPIO_LOW : GpioPinState::GPIO_HIGH;
    }

    class DummyGpioPin : public RaspiGpioPin 
    {
    public:
        DummyGpioPin() : RaspiGpioPin { 0, GpioMode::GPIO_INPUT } {}

        void setState(GpioPinState state) override {}

        GpioPinState getState() override {
            // This should really never be called since the code might rely on the returned value to be accurate.
            STA_UNREACHABLE(); 
        }
    };

    static DummyGpioPin dummyGpio;

    RaspiGpioPin * RaspiGpioPin::DUMMY_GPIO = &dummyGpio;
} // namespace sta


#endif // STA_ARDUINO_GPIO_ENABLED
